@@ -1,19 +1,19 @@
 import base64
 import os
 
-import misaka as m
-from flask import (render_template, flash, abort, request, redirect,
-                   jsonify, url_for, send_from_directory)
+from flask import (render_template, flash, abort, request, redirect, jsonify,
+                   send_from_directory, g, url_for as _url_for)
 from flask_googlelogin import UserMixin, login_user, logout_user, current_user
 from werkzeug import secure_filename
+import misaka as m
 
-from .wsgi import app
+from .constants import (DEFAULT_LANG, DEFAULT_PLATFORM, AVAILABLE_PLATFORMS,
+                       ALLOWED_EXTS)
 from .extensions import login, db
 from .forms import CategoryForm, QuestionForm
+from .helpers import url_for
 from .models import Category, Question
-
-
-ALLOWED_EXT = set(['.png', '.jpg', '.jpeg', '.gif'])
+from .wsgi import app
 
 
 # Setup extensions and hooks
@@ -23,16 +23,44 @@ login.init_app(app)
 db.init_app(app)
 
 
+@app.before_request
+def detect_platform():
+    user_agent = request.headers.get('User-Agent').lower()
+    if 'windows' in user_agent:
+        g.detected_platform = 'win'
+    elif 'macintosh' in user_agent:
+        g.detected_platform = 'mac'
+    elif 'linux' in user_agent:
+        g.detected_platform = 'linux'
+
+
 @app.teardown_request
-def teardown_request(exception=None):
-  if not exception:
-    db.session.commit()
+def commit_db(exception=None):
+    if exception:
+        db.session.rollback()
+    else:
+        db.session.commit()
 
 
 @app.context_processor
 def common():
-    return dict(m=m, current_user=current_user,
-                categories=Category.query.order_by(Category.title).all())
+    params = request.view_args
+    available_platforms = []
+    for platform, platform_title in AVAILABLE_PLATFORMS:
+        if platform == DEFAULT_PLATFORM:
+            continue
+        params['platform'] = platform
+        available_platforms.append((platform, platform_title,
+                                    _url_for(request.endpoint, **params)))
+    return dict(
+        m=m,
+        url_for=url_for,
+        current_user=current_user,
+        selected_lang=request.args.get('lang', DEFAULT_LANG),
+        selected_platform=request.args.get('platform', g.detected_platform),
+        available_platforms=available_platforms,
+        categories=Category.query.order_by(Category.title).all(),
+        )
 
 
 @login.user_loader
@@ -76,8 +104,7 @@ def index():
 def new_category():
     form = CategoryForm(prefix='category')
     if current_user.is_authenticated() and form.validate_on_submit():
-        category = Category()
-        form.populate_obj(category)
+        category = Category.create(form.title.data)
         db.session.add(category)
         db.session.flush()
         flash("Category saved")
@@ -87,27 +114,52 @@ def new_category():
 
 @app.route('/new_question', methods=['GET', 'POST'])
 def new_question():
-    form = QuestionForm(prefix='question')
+    form_args = dict(prefix='question')
+
+    category_id = request.args.get('category_id')
+    eid = request.args.get('eid')
+    if category_id and eid:
+        base_question = Question.query.filter_by(category_id=category_id,
+                                                 eid=eid).first()
+        if base_question.platform == DEFAULT_PLATFORM:
+            abort(400)
+        if not base_question:
+            abort(404)
+        form_args['obj'] = base_question
+
+    form = QuestionForm(**form_args)
+
     form.category_id.choices = [(c.id, c.title) for c in
                                 Category.query.order_by(Category.title).all()]
     if current_user.is_authenticated() and form.validate_on_submit():
         question = Question()
+        question.eid = eid
+        question.creator = current_user.id
         form.populate_obj(question)
         db.session.add(question)
         db.session.flush()
         flash("Question saved")
-        return redirect(question.url())
+        return redirect(question.url(platform=question.platform))
+    else:
+        if request.method == 'POST':
+            print form.errors
+
     return render_template('new_question.html', form=form)
 
 
-@app.route('/<category_title>/<category_id>', methods=['GET', 'POST'])
-def category(category_title, category_id):
+@app.route('/categories/<category_id>', methods=['GET', 'POST', 'DELETE'])
+def category(category_id):
     category = Category.query.get_or_404(category_id)
-    questions = category.get_questions()
+    if request.method == 'DELETE':
+        Question.query.filter_by(category_id=category.id).delete()
+        db.session.delete(category)
+        return jsonify(success=True)
+    selected_platform = request.args.get('platform', g.detected_platform)
+    questions = category.get_questions(platform=selected_platform)
     form = CategoryForm(prefix='category', obj=category)
     if current_user.is_authenticated():
         if request.args.get('edit'):
-            return render_template('edit_category.html', form=form)
+            return render_template('category_form.html', form=form)
         if form.validate_on_submit():
             form.populate_obj(category)
             db.session.add(category)
@@ -118,26 +170,40 @@ def category(category_title, category_id):
                            questions=questions)
 
 
-@app.route('/<category_title>/<category_id>/<question_title>/<question_id>',
-           methods=['GET', 'POST'])
-def question(category_title, category_id, question_title, question_id):
+@app.route('/categories/<category_id>/questions/<eid>/<title>',
+           methods=['GET', 'POST', 'DELETE'])
+def question(category_id, eid, title):
     category = Category.query.get_or_404(category_id)
-    question = Question.query.filter_by(category_id=category_id,
-                                        id=question_id).first()
+
+    selected_lang = request.args.get('lang', DEFAULT_LANG)
+    selected_platform = request.args.get('platform', g.detected_platform)
+    question = (Question.query
+                .filter_by(category_id=category_id, eid=eid,
+                           lang=selected_lang)
+                .filter(Question.platform.in_([DEFAULT_PLATFORM,
+                                               selected_platform]))
+                .first())
     if not question:
         abort(404)
+
+    if request.method == 'DELETE':
+        db.session.delete(question)
+        return jsonify(success=True)
+
     form = QuestionForm(prefix='question', obj=question)
     form.category_id.choices = [(c.id, c.title) for c in
                                 Category.query.order_by(Category.title).all()]
+
     if current_user.is_authenticated():
         if request.args.get('edit'):
-            return render_template('edit_question.html', form=form)
+            return render_template('question_form.html', form=form)
         if form.validate_on_submit():
             form.populate_obj(question)
             db.session.add(question)
             db.session.flush()
             flash("Question saved")
-            return redirect(question.url())
+            return redirect(question.url(platform=question.platform))
+
     return render_template('question.html', category=category,
                            question=question)
 
@@ -151,7 +217,7 @@ def upload():
             filename = (base64.urlsafe_b64encode(os.urandom(9)) + '_' +
                         secure_filename(f.filename))
             # Check if has valid extension
-            if any(map(filename.endswith, ALLOWED_EXT)):
+            if any(map(filename.endswith, ALLOWED_EXTS)):
                 # Store
                 f.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
                 # Return upload path
